@@ -2,19 +2,19 @@
 /*
  * GData Client
  * Copyright (C) Philip Withnall 2009 <philip@tecnocode.co.uk>
- * 
- * GData Client is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ *
+ * GData Client is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * GData Client is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GData Client.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with GData Client.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -36,6 +36,7 @@
 
 #include "gdata-query.h"
 #include "gdata-private.h"
+#include "gdata-types.h"
 
 typedef enum {
 	GDATA_QUERY_PARAM_Q = 1 << 0,
@@ -46,7 +47,7 @@ typedef enum {
 	GDATA_QUERY_PARAM_PUBLISHED_MIN = 1 << 5,
 	GDATA_QUERY_PARAM_PUBLISHED_MAX = 1 << 6,
 	GDATA_QUERY_PARAM_START_INDEX = 1 << 7,
-	GDATA_QUERY_PARAM_STRICT = 1 << 8,
+	GDATA_QUERY_PARAM_IS_STRICT = 1 << 8,
 	GDATA_QUERY_PARAM_MAX_RESULTS = 1 << 9,
 	GDATA_QUERY_PARAM_ENTRY_ID = 1 << 10,
 	GDATA_QUERY_PARAM_ALL = (1 << 11) - 1
@@ -55,6 +56,7 @@ typedef enum {
 static void gdata_query_finalize (GObject *object);
 static void gdata_query_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
 static void gdata_query_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
+static void get_query_uri (GDataQuery *self, const gchar *feed_uri, GString *query_uri, gboolean *params_started);
 
 struct _GDataQueryPrivate {
 	guint parameter_mask; /* GDataQueryParam */
@@ -68,7 +70,7 @@ struct _GDataQueryPrivate {
 	GTimeVal published_min;
 	GTimeVal published_max;
 	gint start_index;
-	gboolean strict;
+	gboolean is_strict;
 	gint max_results;
 	gchar *entry_id;
 
@@ -76,6 +78,8 @@ struct _GDataQueryPrivate {
 	gchar *previous_uri;
 	gboolean use_next_uri;
 	gboolean use_previous_uri;
+
+	gchar *etag;
 };
 
 enum {
@@ -87,9 +91,10 @@ enum {
 	PROP_PUBLISHED_MIN,
 	PROP_PUBLISHED_MAX,
 	PROP_START_INDEX,
-	PROP_STRICT,
+	PROP_IS_STRICT,
 	PROP_MAX_RESULTS,
-	PROP_ENTRY_ID
+	PROP_ENTRY_ID,
+	PROP_ETAG
 };
 
 G_DEFINE_TYPE (GDataQuery, gdata_query, G_TYPE_OBJECT)
@@ -106,6 +111,8 @@ gdata_query_class_init (GDataQueryClass *klass)
 	gobject_class->get_property = gdata_query_get_property;
 	gobject_class->finalize = gdata_query_finalize;
 
+	klass->get_query_uri = get_query_uri;
+
 	/**
 	 * GDataQuery:q:
 	 *
@@ -180,7 +187,7 @@ gdata_query_class_init (GDataQueryClass *klass)
 	g_object_class_install_property (gobject_class, PROP_UPDATED_MIN,
 				g_param_spec_boxed ("updated-min",
 					"Minimum update date", "Minimum date for updates on returned entries.",
-					G_TYPE_TIME_VAL,
+					GDATA_TYPE_G_TIME_VAL,
 					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
@@ -191,7 +198,7 @@ gdata_query_class_init (GDataQueryClass *klass)
 	g_object_class_install_property (gobject_class, PROP_UPDATED_MAX,
 				g_param_spec_boxed ("updated-max",
 					"Maximum update date", "Maximum date for updates on returned entries.",
-					G_TYPE_TIME_VAL,
+					GDATA_TYPE_G_TIME_VAL,
 					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
@@ -202,7 +209,7 @@ gdata_query_class_init (GDataQueryClass *klass)
 	g_object_class_install_property (gobject_class, PROP_PUBLISHED_MIN,
 				g_param_spec_boxed ("published-min",
 					"Minimum publish date", "Minimum date for returned entries to be published.",
-					G_TYPE_TIME_VAL,
+					GDATA_TYPE_G_TIME_VAL,
 					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
@@ -213,7 +220,7 @@ gdata_query_class_init (GDataQueryClass *klass)
 	g_object_class_install_property (gobject_class, PROP_PUBLISHED_MAX,
 				g_param_spec_boxed ("published-max",
 					"Maximum publish date", "Maximum date for returned entries to be published.",
-					G_TYPE_TIME_VAL,
+					GDATA_TYPE_G_TIME_VAL,
 					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
@@ -229,13 +236,15 @@ gdata_query_class_init (GDataQueryClass *klass)
 					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
-	 * GDataQuery:strict:
+	 * GDataQuery:is-strict:
 	 *
 	 * Strict query parameter checking. If this is enabled, an error will be returned by the online service if a parameter is
 	 * not recognised.
+	 *
+	 * Since: 0.2.0
 	 **/
-	g_object_class_install_property (gobject_class, PROP_STRICT,
-				g_param_spec_boolean ("strict",
+	g_object_class_install_property (gobject_class, PROP_IS_STRICT,
+				g_param_spec_boolean ("is-strict",
 					"Strict?", "Should the server be strict about the query?",
 					FALSE,
 					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
@@ -262,6 +271,20 @@ gdata_query_class_init (GDataQueryClass *klass)
 					"Entry ID", "A specific entry ID to return.",
 					NULL,
 					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+	/**
+	 * GDataQuery:etag:
+	 *
+	 * The ETag against which to check for updates. If the server-side ETag matches this one, the requested feed hasn't changed, and is not
+	 * returned unnecessarily.
+	 *
+	 * Since: 0.2.0
+	 **/
+	g_object_class_install_property (gobject_class, PROP_ETAG,
+				g_param_spec_string ("etag",
+					"ETag", "An ETag against which to check.",
+					NULL,
+					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -285,6 +308,8 @@ gdata_query_finalize (GObject *object)
 	g_free (priv->next_uri);
 	g_free (priv->previous_uri);
 
+	g_free (priv->etag);
+
 	/* Chain up to the parent class */
 	G_OBJECT_CLASS (gdata_query_parent_class)->finalize (object);
 }
@@ -319,8 +344,8 @@ gdata_query_get_property (GObject *object, guint property_id, GValue *value, GPa
 		case PROP_START_INDEX:
 			g_value_set_int (value, priv->start_index);
 			break;
-		case PROP_STRICT:
-			g_value_set_boolean (value, priv->strict);
+		case PROP_IS_STRICT:
+			g_value_set_boolean (value, priv->is_strict);
 			break;
 		case PROP_MAX_RESULTS:
 			g_value_set_int (value, priv->max_results);
@@ -328,6 +353,9 @@ gdata_query_get_property (GObject *object, guint property_id, GValue *value, GPa
 		case PROP_ENTRY_ID:
 			g_value_set_string (value, priv->entry_id);
 			break;
+		case PROP_ETAG:
+			g_value_set_string (value, priv->etag);
+			break;
 		default:
 			/* We don't have any other property... */
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -365,8 +393,8 @@ gdata_query_set_property (GObject *object, guint property_id, const GValue *valu
 		case PROP_START_INDEX:
 			gdata_query_set_start_index (self, g_value_get_int (value));
 			break;
-		case PROP_STRICT:
-			gdata_query_set_strict (self, g_value_get_boolean (value));
+		case PROP_IS_STRICT:
+			gdata_query_set_is_strict (self, g_value_get_boolean (value));
 			break;
 		case PROP_MAX_RESULTS:
 			gdata_query_set_max_results (self, g_value_get_int (value));
@@ -374,6 +402,9 @@ gdata_query_set_property (GObject *object, guint property_id, const GValue *valu
 		case PROP_ENTRY_ID:
 			gdata_query_set_entry_id (self, g_value_get_string (value));
 			break;
+		case PROP_ETAG:
+			gdata_query_set_etag (self, g_value_get_string (value));
+			break;
 		default:
 			/* We don't have any other property... */
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -381,6 +412,102 @@ gdata_query_set_property (GObject *object, guint property_id, const GValue *valu
 	}
 }
 
+static void
+get_query_uri (GDataQuery *self, const gchar *feed_uri, GString *query_uri, gboolean *params_started)
+{
+	GDataQueryPrivate *priv = self->priv;
+
+	#define APPEND_SEP g_string_append_c (query_uri, (*params_started == FALSE) ? '?' : '&'); *params_started = TRUE;
+
+	/* Check to see if any parameters have been set */
+	if ((priv->parameter_mask & GDATA_QUERY_PARAM_ALL) == 0)
+		return;
+
+	/* If we've been provided with an entry ID, only append that */
+	if (priv->entry_id != NULL) {
+		g_string_append_c (query_uri, '/');
+		g_string_append_uri_escaped (query_uri, priv->entry_id, NULL, FALSE);
+		return;
+	}
+
+	if (priv->categories != NULL) {
+		g_string_append (query_uri, "/-/");
+		g_string_append_uri_escaped (query_uri, priv->categories, "/", FALSE);
+	}
+
+	/* If that's it, return */
+	if ((priv->parameter_mask & (GDATA_QUERY_PARAM_ALL ^ GDATA_QUERY_PARAM_ENTRY_ID ^ GDATA_QUERY_PARAM_CATEGORIES)) == 0)
+		return;
+
+	/* q param */
+	if (priv->q != NULL) {
+		APPEND_SEP
+		g_string_append (query_uri, "q=");
+		g_string_append_uri_escaped (query_uri, priv->q, NULL, FALSE);
+	}
+
+	if (priv->author != NULL) {
+		APPEND_SEP
+		g_string_append (query_uri, "author=");
+		g_string_append_uri_escaped (query_uri, priv->author, NULL, FALSE);
+	}
+
+	if (priv->updated_min.tv_sec != 0 || priv->updated_min.tv_usec != 0) {
+		gchar *updated_min;
+
+		APPEND_SEP
+		g_string_append (query_uri, "updated-min=");
+		updated_min = g_time_val_to_iso8601 (&(priv->updated_min));
+		g_string_append (query_uri, updated_min);
+		g_free (updated_min);
+	}
+
+	if (priv->updated_max.tv_sec != 0 || priv->updated_max.tv_usec != 0) {
+		gchar *updated_max;
+
+		APPEND_SEP
+		g_string_append (query_uri, "updated-max=");
+		updated_max = g_time_val_to_iso8601 (&(priv->updated_max));
+		g_string_append (query_uri, updated_max);
+		g_free (updated_max);
+	}
+
+	if (priv->published_min.tv_sec != 0 || priv->published_min.tv_usec != 0) {
+		gchar *published_min;
+
+		APPEND_SEP
+		g_string_append (query_uri, "published-min=");
+		published_min = g_time_val_to_iso8601 (&(priv->published_min));
+		g_string_append (query_uri, published_min);
+		g_free (published_min);
+	}
+
+	if (priv->published_max.tv_sec != 0 || priv->published_max.tv_usec != 0) {
+		gchar *published_max;
+
+		APPEND_SEP
+		g_string_append (query_uri, "published-max=");
+		published_max = g_time_val_to_iso8601 (&(priv->published_max));
+		g_string_append (query_uri, published_max);
+		g_free (published_max);
+	}
+
+	if (priv->start_index > 0) {
+		APPEND_SEP
+		g_string_append_printf (query_uri, "start-index=%d", priv->start_index);
+	}
+
+	if (priv->is_strict == TRUE) {
+		APPEND_SEP
+		g_string_append (query_uri, "strict=true");
+	}
+
+	if (priv->max_results > 0) {
+		APPEND_SEP
+		g_string_append_printf (query_uri, "max-results=%d", priv->max_results);
+	}
+}
+
 /**
  * gdata_query_new:
  * @q: a query string
@@ -445,110 +572,25 @@ gdata_query_new_for_id (const gchar *entry_id)
 gchar *
 gdata_query_get_query_uri (GDataQuery *self, const gchar *feed_uri)
 {
-	GDataQueryPrivate *priv = self->priv;
+	GDataQueryClass *klass;
 	GString *query_uri;
-	gboolean params_started = FALSE;
-
-	#define APPEND_SEP g_string_append_c (query_uri, (params_started == FALSE) ? '?' : '&'); params_started = TRUE;
+	gboolean params_started;
 
 	/* Check to see if we're paginating first */
-	if (priv->use_next_uri == TRUE)
-		return g_strdup (priv->next_uri);
-	if (priv->use_previous_uri == TRUE)
-		return g_strdup (priv->previous_uri);
+	if (self->priv->use_next_uri == TRUE)
+		return g_strdup (self->priv->next_uri);
+	if (self->priv->use_previous_uri == TRUE)
+		return g_strdup (self->priv->previous_uri);
 
-	/* Check to see if any parameters have been set */
-	if ((priv->parameter_mask & GDATA_QUERY_PARAM_ALL) == 0)
-		return g_strdup (feed_uri);
-
-	query_uri = g_string_new (feed_uri);
-
-	/* If we've been provided with an entry ID, only append that */
-	if (priv->entry_id != NULL) {
-		g_string_append_c (query_uri, '/');
-		g_string_append_uri_escaped (query_uri, priv->entry_id, NULL, TRUE);
-		return g_string_free (query_uri, FALSE);
-	}
-
-	if (priv->categories != NULL) {
-		g_string_append (query_uri, "/-/");
-		g_string_append_uri_escaped (query_uri, priv->categories, "/", TRUE);
-	}
-
-	/* If that's it, return */
-	if ((priv->parameter_mask & (GDATA_QUERY_PARAM_ALL ^ GDATA_QUERY_PARAM_ENTRY_ID ^ GDATA_QUERY_PARAM_CATEGORIES)) == 0)
-		return g_string_free (query_uri, FALSE);
+	klass = GDATA_QUERY_GET_CLASS (self);
+	g_assert (klass->get_query_uri != NULL);
 
 	/* Determine whether the first param has already been appended (e.g. it exists in the feed_uri) */
 	params_started = (strstr (feed_uri, "?") != NULL) ? TRUE : FALSE;
 
-	/* q param */
-	if (priv->q != NULL) {
-		APPEND_SEP
-		g_string_append (query_uri, "q=");
-		g_string_append_uri_escaped (query_uri, priv->q, NULL, TRUE);
-	}
-
-	if (priv->author != NULL) {
-		APPEND_SEP
-		g_string_append (query_uri, "author=");
-		g_string_append_uri_escaped (query_uri, priv->author, NULL, TRUE);
-	}
-
-	if (priv->updated_min.tv_sec != 0 || priv->updated_min.tv_usec != 0) {
-		gchar *updated_min;
-
-		APPEND_SEP
-		g_string_append (query_uri, "updated-min=");
-		updated_min = g_time_val_to_iso8601 (&(priv->updated_min));
-		g_string_append (query_uri, updated_min);
-		g_free (updated_min);
-	}
-
-	if (priv->updated_max.tv_sec != 0 || priv->updated_max.tv_usec != 0) {
-		gchar *updated_max;
-
-		APPEND_SEP
-		g_string_append (query_uri, "updated-max=");
-		updated_max = g_time_val_to_iso8601 (&(priv->updated_max));
-		g_string_append (query_uri, updated_max);
-		g_free (updated_max);
-	}
-
-	if (priv->published_min.tv_sec != 0 || priv->published_min.tv_usec != 0) {
-		gchar *published_min;
-
-		APPEND_SEP
-		g_string_append (query_uri, "published-min=");
-		published_min = g_time_val_to_iso8601 (&(priv->published_min));
-		g_string_append (query_uri, published_min);
-		g_free (published_min);
-	}
-
-	if (priv->published_max.tv_sec != 0 || priv->published_max.tv_usec != 0) {
-		gchar *published_max;
-
-		APPEND_SEP
-		g_string_append (query_uri, "published-max=");
-		published_max = g_time_val_to_iso8601 (&(priv->published_max));
-		g_string_append (query_uri, published_max);
-		g_free (published_max);
-	}
-
-	if (priv->start_index > 0) {
-		APPEND_SEP
-		g_string_append_printf (query_uri, "start-index=%d", priv->start_index);
-	}
-
-	if (priv->strict == TRUE) {
-		APPEND_SEP
-		g_string_append (query_uri, "strict=true");
-	}
-
-	if (priv->max_results > 0) {
-		APPEND_SEP
-		g_string_append_printf (query_uri, "max-results=%d", priv->max_results);
-	}
+	/* Build the query URI */
+	query_uri = g_string_new (feed_uri);
+	klass->get_query_uri (self, feed_uri, query_uri, &params_started);
 
 	return g_string_free (query_uri, FALSE);
 }
@@ -586,7 +628,7 @@ gdata_query_set_q (GDataQuery *self, const gchar *q)
 	self->priv->q = g_strdup (q);
 
 	if (q == NULL)
-		self->priv->parameter_mask &= (GDATA_QUERY_PARAM_ALL ^ GDATA_QUERY_PARAM_Q);
+		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_Q;
 	else
 		self->priv->parameter_mask |= GDATA_QUERY_PARAM_Q;
 
@@ -626,7 +668,7 @@ gdata_query_set_categories (GDataQuery *self, const gchar *categories)
 	self->priv->categories = g_strdup (categories);
 
 	if (categories == NULL)
-		self->priv->parameter_mask &= (GDATA_QUERY_PARAM_ALL ^ GDATA_QUERY_PARAM_CATEGORIES);
+		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_CATEGORIES;
 	else
 		self->priv->parameter_mask |= GDATA_QUERY_PARAM_CATEGORIES;
 
@@ -666,7 +708,7 @@ gdata_query_set_author (GDataQuery *self, const gchar *author)
 	self->priv->author = g_strdup (author);
 
 	if (author == NULL)
-		self->priv->parameter_mask &= (GDATA_QUERY_PARAM_ALL ^ GDATA_QUERY_PARAM_AUTHOR);
+		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_AUTHOR;
 	else
 		self->priv->parameter_mask |= GDATA_QUERY_PARAM_AUTHOR;
 
@@ -679,7 +721,7 @@ gdata_query_set_author (GDataQuery *self, const gchar *author)
  * @updated_min: a #GTimeVal
  *
  * Gets the #GDataQuery:updated-min property and puts it in @updated_min. If the property is unset,
- * both fields in the #GTimeVal will be set to 0.
+ * both fields in the #GTimeVal will be set to %0.
  **/
 void
 gdata_query_get_updated_min (GDataQuery *self, GTimeVal *updated_min)
@@ -706,7 +748,7 @@ gdata_query_set_updated_min (GDataQuery *self, GTimeVal *updated_min)
 	if (updated_min == NULL) {
 		self->priv->updated_min.tv_sec = 0;
 		self->priv->updated_min.tv_usec = 0;
-		self->priv->parameter_mask &= (GDATA_QUERY_PARAM_ALL ^ GDATA_QUERY_PARAM_UPDATED_MIN);
+		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_UPDATED_MIN;
 	} else {
 		self->priv->updated_min = *updated_min;
 		self->priv->parameter_mask |= GDATA_QUERY_PARAM_UPDATED_MIN;
@@ -721,7 +763,7 @@ gdata_query_set_updated_min (GDataQuery *self, GTimeVal *updated_min)
  * @updated_max: a #GTimeVal
  *
  * Gets the #GDataQuery:updated-max property and puts it in @updated_max. If the property is unset,
- * both fields in the #GTimeVal will be set to 0.
+ * both fields in the #GTimeVal will be set to %0.
  **/
 void
 gdata_query_get_updated_max (GDataQuery *self, GTimeVal *updated_max)
@@ -748,7 +790,7 @@ gdata_query_set_updated_max (GDataQuery *self, GTimeVal *updated_max)
 	if (updated_max == NULL) {
 		self->priv->updated_max.tv_sec = 0;
 		self->priv->updated_max.tv_usec = 0;
-		self->priv->parameter_mask &= (GDATA_QUERY_PARAM_ALL ^ GDATA_QUERY_PARAM_UPDATED_MAX);
+		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_UPDATED_MAX;
 	} else {
 		self->priv->updated_max = *updated_max;
 		self->priv->parameter_mask |= GDATA_QUERY_PARAM_UPDATED_MAX;
@@ -763,7 +805,7 @@ gdata_query_set_updated_max (GDataQuery *self, GTimeVal *updated_max)
  * @published_min: a #GTimeVal
  *
  * Gets the #GDataQuery:published-min property and puts it in @published_min. If the property is unset,
- * both fields in the #GTimeVal will be set to 0.
+ * both fields in the #GTimeVal will be set to %0.
  **/
 void
 gdata_query_get_published_min (GDataQuery *self, GTimeVal *published_min)
@@ -790,7 +832,7 @@ gdata_query_set_published_min (GDataQuery *self, GTimeVal *published_min)
 	if (published_min == NULL) {
 		self->priv->published_min.tv_sec = 0;
 		self->priv->published_min.tv_usec = 0;
-		self->priv->parameter_mask &= (GDATA_QUERY_PARAM_ALL ^ GDATA_QUERY_PARAM_PUBLISHED_MIN);
+		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_PUBLISHED_MIN;
 	} else {
 		self->priv->published_min = *published_min;
 		self->priv->parameter_mask |= GDATA_QUERY_PARAM_PUBLISHED_MIN;
@@ -805,7 +847,7 @@ gdata_query_set_published_min (GDataQuery *self, GTimeVal *published_min)
  * @published_max: a #GTimeVal
  *
  * Gets the #GDataQuery:published-max property and puts it in @published_max. If the property is unset,
- * both fields in the #GTimeVal will be set to 0.
+ * both fields in the #GTimeVal will be set to %0.
  **/
 void
 gdata_query_get_published_max (GDataQuery *self, GTimeVal *published_max)
@@ -832,7 +874,7 @@ gdata_query_set_published_max (GDataQuery *self, GTimeVal *published_max)
 	if (published_max == NULL) {
 		self->priv->published_max.tv_sec = 0;
 		self->priv->published_max.tv_usec = 0;
-		self->priv->parameter_mask &= (GDATA_QUERY_PARAM_ALL ^ GDATA_QUERY_PARAM_PUBLISHED_MAX);
+		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_PUBLISHED_MAX;
 	} else {
 		self->priv->published_max = *published_max;
 		self->priv->parameter_mask |= GDATA_QUERY_PARAM_PUBLISHED_MAX;
@@ -878,7 +920,7 @@ gdata_query_set_start_index (GDataQuery *self, gint start_index)
 	self->priv->start_index = start_index;
 
 	if (start_index == -1)
-		self->priv->parameter_mask &= (GDATA_QUERY_PARAM_ALL ^ GDATA_QUERY_PARAM_START_INDEX);
+		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_START_INDEX;
 	else
 		self->priv->parameter_mask |= GDATA_QUERY_PARAM_START_INDEX;
 
@@ -886,40 +928,44 @@ gdata_query_set_start_index (GDataQuery *self, gint start_index)
 }
 
 /**
- * gdata_query_get_strict:
+ * gdata_query_is_strict:
  * @self: a #GDataQuery
  *
- * Gets the #GDataQuery:strict property.
+ * Gets the #GDataQuery:is-strict property.
  *
  * Return value: the strict property
+ *
+ * Since: 0.2.0
  **/
 gboolean
-gdata_query_get_strict (GDataQuery *self)
+gdata_query_is_strict (GDataQuery *self)
 {
 	g_return_val_if_fail (GDATA_IS_QUERY (self), FALSE);
-	return self->priv->strict;
+	return self->priv->is_strict;
 }
 
 /**
- * gdata_query_set_strict:
+ * gdata_query_set_is_strict:
  * @self: a #GDataQuery
- * @strict: the new strict value
+ * @is_strict: the new strict value
  *
- * Sets the #GDataQuery:strict property of the #GDataQuery to the new strict value, @strict.
+ * Sets the #GDataQuery:is-strict property of the #GDataQuery to the new strict value, @is_strict.
+ *
+ * Since: 0.2.0
  **/
 void
-gdata_query_set_strict (GDataQuery *self, gboolean strict)
+gdata_query_set_is_strict (GDataQuery *self, gboolean is_strict)
 {
 	g_return_if_fail (GDATA_IS_QUERY (self));
 
-	self->priv->strict = strict;
+	self->priv->is_strict = is_strict;
 
-	if (strict == FALSE)
-		self->priv->parameter_mask &= (GDATA_QUERY_PARAM_ALL ^ GDATA_QUERY_PARAM_STRICT);
+	if (is_strict == FALSE)
+		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_IS_STRICT;
 	else
-		self->priv->parameter_mask |= GDATA_QUERY_PARAM_STRICT;
+		self->priv->parameter_mask |= GDATA_QUERY_PARAM_IS_STRICT;
 
-	g_object_notify (G_OBJECT (self), "strict");
+	g_object_notify (G_OBJECT (self), "is-strict");
 }
 
 /**
@@ -955,7 +1001,7 @@ gdata_query_set_max_results (GDataQuery *self, gint max_results)
 	self->priv->max_results = max_results;
 
 	if (max_results == -1)
-		self->priv->parameter_mask &= (GDATA_QUERY_PARAM_ALL ^ GDATA_QUERY_PARAM_MAX_RESULTS);
+		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_MAX_RESULTS;
 	else
 		self->priv->parameter_mask |= GDATA_QUERY_PARAM_MAX_RESULTS;
 
@@ -995,13 +1041,51 @@ gdata_query_set_entry_id (GDataQuery *self, const gchar *entry_id)
 	self->priv->entry_id = g_strdup (entry_id);
 
 	if (entry_id == NULL)
-		self->priv->parameter_mask &= (GDATA_QUERY_PARAM_ALL ^ GDATA_QUERY_PARAM_ENTRY_ID);
+		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_ENTRY_ID;
 	else
 		self->priv->parameter_mask |= GDATA_QUERY_PARAM_ENTRY_ID;
 
 	g_object_notify (G_OBJECT (self), "entry-id");
 }
 
+/**
+ * gdata_query_get_etag:
+ * @self: a #GDataQuery
+ *
+ * Gets the #GDataQuery:etag property.
+ *
+ * Return value: the ETag property, or %NULL if it is unset
+ *
+ * Since: 0.2.0
+ **/
+const gchar *
+gdata_query_get_etag (GDataQuery *self)
+{
+	g_return_val_if_fail (GDATA_IS_QUERY (self), NULL);
+	return self->priv->etag;
+}
+
+/**
+ * gdata_query_set_etag:
+ * @self: a #GDataQuery
+ * @etag: the new ETag
+ *
+ * Sets the #GDataQuery:etag property of the #GDataQuery to the new ETag, @etag.
+ *
+ * Set @etag to %NULL to not check against the server-side ETag.
+ *
+ * Since: 0.2.0
+ **/
+void
+gdata_query_set_etag (GDataQuery *self, const gchar *etag)
+{
+	g_return_if_fail (GDATA_IS_QUERY (self));
+
+	g_free (self->priv->etag);
+	self->priv->etag = g_strdup (etag);
+	g_object_notify (G_OBJECT (self), "etag");
+}
+
 void
 _gdata_query_set_next_uri (GDataQuery *self, const gchar *next_uri)
 {
@@ -1037,12 +1121,18 @@ _gdata_query_set_previous_uri (GDataQuery *self, const gchar *previous_uri)
 void
 gdata_query_next_page (GDataQuery *self)
 {
-	if (self->priv->next_uri != NULL) {
-		self->priv->use_next_uri = TRUE;
-		self->priv->use_previous_uri = FALSE;
+	GDataQueryPrivate *priv = self->priv;
+
+	if (priv->next_uri != NULL) {
+		priv->use_next_uri = TRUE;
+		priv->use_previous_uri = FALSE;
 	} else {
-		self->priv->start_index += self->priv->max_results;
+		priv->start_index += priv->max_results;
 	}
+
+	/* Our current ETag will not be relevant */
+	g_free (priv->etag);
+	priv->etag = NULL;
 }
 
 /**
@@ -1059,14 +1149,20 @@ gdata_query_next_page (GDataQuery *self)
 gboolean
 gdata_query_previous_page (GDataQuery *self)
 {
-	if (self->priv->next_uri != NULL) {
-		self->priv->use_previous_uri = TRUE;
-		self->priv->use_next_uri = FALSE;
-	} else if (self->priv->start_index < self->priv->max_results) {
+	GDataQueryPrivate *priv = self->priv;
+
+	if (priv->next_uri != NULL) {
+		priv->use_previous_uri = TRUE;
+		priv->use_next_uri = FALSE;
+	} else if (priv->start_index < priv->max_results) {
 		return FALSE;
 	} else {
-		self->priv->start_index -= self->priv->max_results;
+		priv->start_index -= priv->max_results;
 	}
 
+	/* Our current ETag will not be relevant */
+	g_free (priv->etag);
+	priv->etag = NULL;
+
 	return TRUE;
 }
