@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * GData Client
- * Copyright (C) Philip Withnall 2009 <philip@tecnocode.co.uk>
+ * Copyright (C) Philip Withnall 2009–2010 <philip@tecnocode.co.uk>
  *
  * GData Client is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -179,9 +179,12 @@ gdata_contacts_contact_init (GDataContactsContact *self)
 	/* Create a default name, so the name's properties can be set for a blank contact */
 	self->priv->name = gdata_gd_name_new (NULL, NULL);
 
-	/* Listen to change notifications for the entry's title, since it's linked to GDataGDName:fullName */
+	/* Listen to change notifications for the entry's title, since it's linked to GDataGDName:full-name */
 	g_signal_connect (self, "notify::title", (GCallback) notify_title_cb, self);
-	g_signal_connect (self->priv->name, "notify::fullName", (GCallback) notify_full_name_cb, self);
+	g_signal_connect (self->priv->name, "notify::full-name", (GCallback) notify_full_name_cb, self);
+
+	/* Set the edited property to the current time (creation time) */
+	g_get_current_time (&(self->priv->edited));
 }
 
 static void
@@ -275,21 +278,19 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 		if (g_time_val_from_iso8601 ((gchar*) edited, &(self->priv->edited)) == FALSE) {
 			/* Error */
 			gdata_parser_error_not_iso8601_format (node, (gchar*) edited, error);
-			xmlFree (edited);
+			g_free (edited);
 			return FALSE;
 		}
-		xmlFree (edited);
+		g_free (edited);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "name") == 0) {
 		/* gd:name */
 		GDataGDName *name = GDATA_GD_NAME (_gdata_parsable_new_from_xml_node (GDATA_TYPE_GD_NAME, doc, node, NULL, error));
-
 		if (name == NULL)
 			return FALSE;
 
 		if (self->priv->name != NULL)
 			g_object_unref (self->priv->name);
 		self->priv->name = name;
-		g_object_notify (G_OBJECT (self), "name");
 	} else if (xmlStrcmp (node->name, (xmlChar*) "email") == 0) {
 		/* gd:email */
 		GDataGDEmailAddress *email = GDATA_GD_EMAIL_ADDRESS (_gdata_parsable_new_from_xml_node (GDATA_TYPE_GD_EMAIL_ADDRESS, doc,
@@ -341,17 +342,21 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 		/* Get either the value property, or the element's content */
 		value = xmlGetProp (node, (xmlChar*) "value");
 		if (value == NULL) {
+			xmlNode *child_node;
+
 			/* Use the element's content instead (arbitrary XML) */
 			buffer = xmlBufferCreate ();
-			xmlNodeDump (buffer, doc, node, 0, 0);
+			for (child_node = node->children; child_node != NULL; child_node = child_node->next)
+				xmlNodeDump (buffer, doc, child_node, 0, 0);
 			value = (xmlChar*) xmlBufferContent (buffer);
-			xmlBufferFree (buffer);
 		}
 
 		gdata_contacts_contact_set_extended_property (self, (gchar*) name, (gchar*) value);
 
-		if (buffer == NULL)
-			xmlFree (value);
+		if (buffer != NULL)
+			xmlBufferFree (buffer);
+		else
+			g_free (value);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "groupMembershipInfo") == 0) {
 		/* gContact:groupMembershipInfo */
 		xmlChar *href, *deleted;
@@ -369,14 +374,13 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 			deleted_bool = TRUE;
 		else {
 			gdata_parser_error_unknown_property_value (node, "deleted", (gchar*) deleted, error);
-			xmlFree (deleted);
+			g_free (deleted);
 			return FALSE;
 		}
-		xmlFree (deleted);
+		g_free (deleted);
 
 		/* Insert it into the hash table */
-		g_hash_table_insert (self->priv->groups, g_strdup ((gchar*) href), GUINT_TO_POINTER (deleted_bool));
-		xmlFree (href);
+		g_hash_table_insert (self->priv->groups, (gchar*) href, GUINT_TO_POINTER (deleted_bool));
 	} else if (xmlStrcmp (node->name, (xmlChar*) "deleted") == 0) {
 		/* gd:deleted */
 		self->priv->deleted = TRUE;
@@ -385,13 +389,9 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 		if (self->priv->photo_etag == NULL && xmlStrcmp (node->name, (xmlChar*) "link") == 0) {
 			xmlChar *rel = xmlGetProp (node, (xmlChar*) "rel");
 			if (xmlStrcmp (rel, (xmlChar*) "http://schemas.google.com/contacts/2008/rel#photo") == 0) {
-				xmlChar *etag;
-
 				/* It's the photo link (http://code.google.com/apis/contacts/docs/2.0/reference.html#Photos), whose ETag we should
 				 * note down, then pass onto the parent class to parse properly */
-				etag = xmlGetProp (node, (xmlChar*) "etag");
-				self->priv->photo_etag = g_strdup ((gchar*) etag);
-				xmlFree (etag);
+				self->priv->photo_etag = (gchar*) xmlGetProp (node, (xmlChar*) "etag");
 			}
 			xmlFree (rel);
 		}
@@ -1170,7 +1170,7 @@ gdata_contacts_contact_has_photo (GDataContactsContact *self)
  * If @cancellable is not %NULL, then the operation can be cancelled by triggering the @cancellable object from another thread.
  * If the operation was cancelled, the error %G_IO_ERROR_CANCELLED will be returned.
  *
- * If there is an error getting the photo, a %GDATA_SERVICE_ERROR_WITH_QUERY error will be returned.
+ * If there is an error getting the photo, a %GDATA_SERVICE_ERROR_PROTOCOL_ERROR error will be returned.
  *
  * Return value: the image data, or %NULL; free with g_free()
  *
@@ -1221,8 +1221,8 @@ gdata_contacts_contact_get_photo (GDataContactsContact *self, GDataContactsServi
 	if (status != 200) {
 		/* Error */
 		g_assert (klass->parse_error_response != NULL);
-		klass->parse_error_response (GDATA_SERVICE (service), GDATA_SERVICE_ERROR_WITH_QUERY, status, message->reason_phrase,
-					     message->response_body->data, message->response_body->length, error);
+		klass->parse_error_response (GDATA_SERVICE (service), GDATA_OPERATION_DOWNLOAD, status, message->reason_phrase,
+		                             message->response_body->data, message->response_body->length, error);
 		g_object_unref (message);
 		return NULL;
 	}
@@ -1257,7 +1257,7 @@ gdata_contacts_contact_get_photo (GDataContactsContact *self, GDataContactsServi
  * If @cancellable is not %NULL, then the operation can be cancelled by triggering the @cancellable object from another thread.
  * If the operation was cancelled, the error %G_IO_ERROR_CANCELLED will be returned.
  *
- * If there is an error setting the photo, a %GDATA_SERVICE_ERROR_WITH_UPDATE error will be returned.
+ * If there is an error setting the photo, a %GDATA_SERVICE_ERROR_PROTOCOL_ERROR error will be returned.
  *
  * Return value: %TRUE on success, %FALSE otherwise
  *
@@ -1320,8 +1320,8 @@ gdata_contacts_contact_set_photo (GDataContactsContact *self, GDataService *serv
 	if (status != 200) {
 		/* Error */
 		g_assert (klass->parse_error_response != NULL);
-		klass->parse_error_response (service, GDATA_SERVICE_ERROR_WITH_UPDATE, status, message->reason_phrase, message->response_body->data,
-					     message->response_body->length, error);
+		klass->parse_error_response (service, GDATA_OPERATION_UPLOAD, status, message->reason_phrase, message->response_body->data,
+		                             message->response_body->length, error);
 		g_object_unref (message);
 		return FALSE;
 	}
